/*****************************************************************************
* Time Bomb example (FSM)
*
* Copyright (c) 2006 Miro Samek. All rights reserved.
*
* Email: miro@quantum-leaps.com
* Web:   www.quantum-leaps.com
*****************************************************************************/
#include "qpn_port.h"
#include "qassert.h"
#include "bsp.h"
#include "bomb.h"

/*--------------------------------------------------------------------------*/
void Pulse_rampUp(Pulse *me);
void Pulse_rampDown(Pulse *me);

/*..........................................................................*/
void Pulse_init(Pulse *me, uint8_t freq) {
    me->timeout__ = BSP_TICKS_PER_SEC / freq;              /* the ramp tout */
    QFsm_init((QFsm *)me, (QState)&Pulse_rampUp);/*trigger the initial tran.*/
}
/*..........................................................................*/
void Pulse_rampUp(Pulse *me) {
    switch (Q_SIG(me)) {
        case Q_ENTRY_SIG: {
            QActive_arm((QActive *)me, me->timeout__);
            me->brightness__ = 1;
            LED_grn(me->brightness__);
            break;
        }
        case Q_EXIT_SIG: {
            QF_post(BOMB_PRIO, ARM_SIG, 0);
            break;
        }
        case Q_TIMEOUT_SIG: {
            if (me->brightness__ < (uint8_t)(1 << 7)) {
                QActive_arm((QActive *)me, me->timeout__);
                me->brightness__ <<= 1;
                LED_grn(me->brightness__);
            }
            else {
                Q_TRAN(&Pulse_rampDown);
            }
            break;
        }
    }
}
/*..........................................................................*/
void Pulse_rampDown(Pulse *me) {
    switch (Q_SIG(me)) {
        case Q_ENTRY_SIG: {
            QActive_arm((QActive *)me, me->timeout__);
            me->brightness__ = (uint8_t)(1 << 7);
            LED_grn(me->brightness__);
            break;
        }
        case Q_TIMEOUT_SIG: {
            if (me->brightness__ > (uint8_t)1) {
                me->brightness__ >>= 1;
                LED_grn(me->brightness__);
                QActive_arm((QActive *)me, me->timeout__);
            }
            else {
                Q_TRAN(&Pulse_rampUp);
            }
            break;
        }
    }
}
