/*****************************************************************************
* Time Bomb example (FSM)
*
* Copyright (c) 2006 Miro Samek. All rights reserved.
*
* Email: miro@quantum-leaps.com
* Web:   www.quantum-leaps.com
*****************************************************************************/
#include "qpn_port.h"
#include "qassert.h"
#include "bsp.h"

sfr16 TMR2RL = 0xCA;                                 /* Timer2 reload value */
sfr16 TMR2   = 0xCC;                                      /* Timer2 counter */

/*..........................................................................*/
void timer2_ISR (void) interrupt 5 {
   QF_tick();
   TF2H = 0;                                 /* clear Timer2 interrupt flag */
}
/*..........................................................................*/
void BSP_init(void) {
    OSCICN = 0x07;           /* Set SYSCLK as Int Osc 24.5 MHz (full speed) */
    RSTSRC = 0x04;                         /* Enable missing clock detector */

    PCA0CN = 0x40;                       /* Enable PCA Counter/Timer (PCA0) */
    PCA0MD = 0x00;  /* Set PCA timebase = SYSCLK / 12, disable watchdog WDT */

                                     /* Duty cycle = (256 - PCA0CPHn) / 256 */
    PCA0CPH0 = 0x00;       /* Config Mod0 PWM for a 100% initial duty cycle */
    PCA0CPH1 = 0x00;       /* Config Mod1 PWM for a 100% initial duty cycle */

    PCA0CPM0 = 0x42;                            /* Enable 8-bit PWM on CEX0 */
    PCA0CPM1 = 0x42;                            /* Enable 8-bit PWM on CEX1 */

    /* Configure the Crossbar and GPIO ports.
    * P0.0  -  Skipped
    * P0.1  -  Skipped
    * P0.2  -  CEX0 (PCA),  Push-Pull,  Digital  (Red LEDs)
    * P0.3  -  CEX1 (PCA),  Push-Pull,  Digital  (Green LEDs)
    * P0.4  -  Unassigned
    * P0.5  -  Unassigned
    * P0.6  -  Unassigned
    * P0.7  -  Unassigned
    */
    P0MDOUT = 0x0C;              /* Config P0.2 & P0.3 as push-pull outputs */
    XBR0    = 0x03;                                     /* Skip P0.0 & P0.1 */
    XBR1    = 0x80;             /* Enable PCA Capture/Compare Outputs 0 & 1 */
    XBR2    = 0x40;                                      /* Enable Crossbar */

    IP      = ~0;   /* set all interrupts to the high priority (no nesting) */

    TMR2CN  = 0x00;                    /* Stop Timer2; Clear TF2, SYSCLK/12 */
    CKCON  &= ~0x60;                      /* Timer2 clocked based on T2XCLK */

    TMR2RL  = 0xFFFF - (BSP_MCK/12)/BSP_TICKS_PER_SEC;      /* reload value */
    TMR2    = TMR2RL;                  /* Initialize Timer2 to reload value */
}
/*..........................................................................*/
void QF_start(void) {
    ET2 = 1;                                    /* Enable Timer2 interrupts */
    TR2 = 1;                                                /* Start Timer2 */
}
/*..........................................................................*/
void QF_onIdle(void) {
    PCON |= 0x01;                                       /* set the IDLE bit */
    PCON = PCON;             /* ... followed by a 3-cycle dummy instruction */
}
/*..........................................................................*/
void Q_assert_handler(char const Q_ROM *file, int line) {
    file = file;                                  /* avoid compiler warning */
    line = line;                                  /* avoid compiler warning */
    QF_INT_LOCK();
    LED_red(255);
    LED_grn(0);
    for (;;) {
    }
}

